@@ -32,7 +32,7 @@ use AnyEvent::RabbitMQ::LocalQueue;
 
 use namespace::clean;
 
-our $VERSION = '1.16';
+our $VERSION = '1.17';
 
 use constant {
     _ST_CLOSED => 0,
@@ -44,6 +44,8 @@ use constant {
 Readonly my $DEFAULT_AMQP_SPEC
     => File::ShareDir::dist_dir("AnyEvent-RabbitMQ") . '/fixed_amqp0-9-1.xml';
 
+Readonly my $DEFAULT_CHANNEL_MAX => 2**16;
+
 sub new {
     my $class = shift;
     return bless {
@@ -55,6 +57,9 @@ sub new {
         _channels          => {},
         _login_user        => '',
         _server_properties => {},
+        _frame_max         => undef,
+        _body_max          => undef,
+        _channel_max       => undef,
     }, $class;
 }
 
@@ -341,9 +346,24 @@ sub _tune {
             my $self = $weak_self or return;
             my $frame = shift;
 
-            my %tune = map { my $t = $args{tune}{$_};
-                             ( $_ => defined($t) ? $t : $frame->method_frame->$_ ) }
-                          qw( channel_max frame_max heartbeat );
+            my %tune;
+            foreach (qw( channel_max frame_max heartbeat )) {
+                my $client = $args{tune}{$_} || 0;
+                my $server = $frame->method_frame->$_ || 0;
+
+                # negotiate with the server such that we cannot request a larger
+                # value set by the server, unless the server said unlimited
+                $tune{$_} = ($server == 0 or $client == 0)
+                    ? ($server > $client ? $server : $client)   # max
+                    : ($client > $server ? $server : $client);  # min
+            }
+
+            if ($self->{_frame_max} = $tune{frame_max}) {
+                # calculate how big the body can actually be
+                $self->{_body_max} = $self->{_frame_max} - Net::AMQP::_HEADER_LEN - Net::AMQP::_FOOTER_LEN;
+            }
+
+            $self->{_channel_max} = $tune{channel_max} || $DEFAULT_CHANNEL_MAX;
 
             $self->_push_write(
                 Net::AMQP::Protocol::Connection::TuneOk->new(%tune,)
@@ -483,8 +503,6 @@ sub _finish_close {
     return;
 }
 
-use constant _MAX_CHANID => 0xFFFF;
-
 sub open_channel {
     my $self = shift;
     my %args = $self->_set_cbs(@_);
@@ -501,8 +519,8 @@ sub open_channel {
 
     if (!$id) {
         my $try_id = $self->{_last_chan_id};
-        for (1 .. 2**16) {
-            $try_id = 1 if ++$try_id > _MAX_CHANID;
+        for (1 .. $self->{_channel_max}) {
+            $try_id = 1 if ++$try_id > $self->{_channel_max};
             unless (defined $self->{_channels}->{$try_id}) {
                 $id = $try_id;
                 last;
